#include "Wps2Nod.h"


waypoint_t *waypoints;
int numWaypoints = 0;
char mapname[32];


int main(int argc, char* argv[])
{
	char outname[256];
	FILE *fWpsFile, *fNodFile;
	bool b;

	if (argc < 2)
	{
		printf("Usage: %s <wps-file> [nod-file]\n", argv[0]);
		return -1;
	}

	if (argc == 3)
		strcpy(outname, argv[2]);
	else
	{
		int pos;
		strcpy(outname, argv[1]);
		pos = strlen(outname) - 3;
		strcpy(outname+pos, "nod");
	}

	if (!(fWpsFile = fopen(argv[1], "rb")))
	{
		printf("Unable to open waypoint file %s\n", argv[1]);
		return -2;
	}

	b = ParseWPSFile(fWpsFile);
	fclose(fWpsFile);

	if (!b)
		return -3;

	if (!(fNodFile = fopen(outname, "wb")))
	{
		printf("Unable to create node file %s\n", outname);
		Cleanup();
		return -4;
	}

	WriteNodFile(fNodFile);
	fclose(fNodFile);

	Cleanup();

	return 0;
}

bool ParseWPSFile(FILE* f)
{
	char buf[256];
	char name[8];
	int ver;

	int i, j;


	fread(name, sizeof(char), 8, f);
	fread(&ver, sizeof(int), 1, f);

	if ((strncmp(name, "kajibot", 7) != 0) || (ver != 1))
	{
		printf("Not a valid wolfbot waypoint file\n");
		return false;
	}

	fread(&numWaypoints, sizeof(int), 1, f);

	fread(buf, sizeof(char), 4, f);

	fread(&mapname, sizeof(char), 32, f);

	waypoints = new waypoint_t[numWaypoints];
	for (i = 0; i < numWaypoints; i++)
	{
		fread(&waypoints[i].pos, sizeof(vec3_t), 1, f);
		fread(&waypoints[i].num, sizeof(int), 1, f);
		fread(&waypoints[i].flags, sizeof(int), 1, f);
		fread(&waypoints[i].entNums[0], sizeof(int), 1, f);
		waypoints[i].entNums[1] = waypoints[i].entNums[0];
		fread(&waypoints[i].entNums[2], sizeof(int), 1, f);
		fread(&waypoints[i].objFlags, sizeof(int), 1, f);
		fread(&waypoints[i].numLinks, sizeof(int), 1, f);
		fread(&waypoints[i].crap, sizeof(int), 1, f);

		waypoints[i].links = new link_t[waypoints[i].numLinks];

		for (j = 0; j < waypoints[i].numLinks; j++)
		{
			fread(&waypoints[i].links[j].num, sizeof(int), 1, f);
			fread(&waypoints[i].links[j].dist, sizeof(float), 1, f);
			fread(&waypoints[i].links[j].flags, sizeof(int), 1, f);
		}
	}

	return true;
}

void WriteNodFile(FILE* f)
{
	char name[] = BOT_MOD_NAME;
	float ver = BOT_MOD_VERSION;
	short flags;
	short int ents[3];
	int i, j;

	fwrite(&name, strlen(name)+1, 1, f);
	fwrite(&ver, sizeof(float), 1, f);
	fwrite(&mapname, strlen(mapname)+1, 1, f);
	fwrite(&numWaypoints, sizeof(short int), 1, f);

	for (i = 0; i < numWaypoints; i++)
	{
		fwrite(&(waypoints[i].num), sizeof(short int), 1, f);
		fwrite(&(waypoints[i].pos), sizeof(vec3_t), 1, f);
		fwrite(&(waypoints[i].flags), sizeof(short int), 1, f);

		GetEnts(i, ents);
		fwrite(ents, sizeof(short int), 3, f);

		fwrite(&(waypoints[i].objFlags), sizeof(int), 1, f);
		fwrite(&(waypoints[i].numLinks), sizeof(short int), 1, f);

		for (j = 0; j < waypoints[i].numLinks; j++)
		{
			fwrite(&(waypoints[i].links[j].num), sizeof(short int), 1, f);

			flags = (waypoints[i].links[j].flags & 0xFF);
			fwrite(&flags, sizeof(short int), 1, f);
		}
	}
}

void Cleanup()
{
	int i;

	for (i = 0; i < numWaypoints; i++)
		delete[] waypoints[i].links;

	delete[] waypoints;
}

void GetEnts(int num, short int* ents)
{
	ents[0] = ((waypoints[num].entNums[0] & 0x000FFFFF) >> 10);
	ents[1] = ((waypoints[num].entNums[1] & 0xFFF00000) >> 20);
	ents[2] = waypoints[num].entNums[2];
}

